#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <unistd.h>

#include <Carbon/Carbon.h>
#include <JavaVM/jni.h>

#define DEBUG 0
#define USE_RunApplicationEventLoop 0

#define CLASSPATH "-Djava.class.path="

static const char *ProgName;
static const char **Args;
static int FirstOptionArg, LastOptionArg, OptionArgCount;
static int FirstMainArg, LastMainArg, MainClassArg;

/* SPI */
extern OSErr CPSEnableForegroundOperation(ProcessSerialNumber *psn,  UInt32 _arg2, UInt32 _arg3, UInt32 _arg4, UInt32 _arg5);

int launchVM();

#if USE_RunApplicationEventLoop

void timerCallback(EventLoopTimerRef inTimer, void *inUserData) {
	launchVM();
	QuitApplicationEventLoop();
}

#endif

int main(int argc, const char **argv)
{
	Args= argv;
	ProgName= argv[0];
	
     // Start with the first arg, not the path to the tool
    int argIndex;
     
    OptionArgCount= 0;
    
    // JVM options start with -
    for (FirstOptionArg= LastOptionArg= argIndex= 1; argIndex < argc; argIndex++) {
        
        char *arg= (char*) argv[argIndex];
                
        if (arg[0] != '-')
        	break;
         
        /* If the option string is '-cp' or '-classpath', replace it with '-Djava.class.path=' and append
           the next option which contains the actual class path.
        */
		if (strcmp(arg, "-cp") == 0 || strcmp(arg, "-classpath") == 0) {
            argv[argIndex++]= NULL;
            if (argIndex < argc) {
            	arg= (char*) argv[argIndex];
            	char *cp= malloc(strlen(CLASSPATH) + strlen(arg) + 1);
            	strcpy(cp, CLASSPATH);
            	strcat(cp, arg);
            	argv[argIndex]= cp;
            } else {
                /* We shouldn't reach here unless the last arg was -cp */
                fprintf(stderr, "%s: Error parsing class path.\n", ProgName);
                /* Release the option CFString here since the break; statement is going */
                /* to skip the release in this loop */
                break;
            }
        }

        /* On to the next one */
         OptionArgCount++;
    }
    LastOptionArg= argIndex;
    
    /* If there are still args left lets get the main class */
    if (argIndex < argc ) {
    	MainClassArg= argIndex++;
	} else { 
		fprintf(stderr, "%s: no main class specified.\n", ProgName);
        exit(-1);
    }
    
    FirstMainArg= argIndex;
    LastMainArg= argc;
    		
	ProcessSerialNumber psn;
		
	GetCurrentProcess(&psn);
	CPSEnableForegroundOperation(&psn, 0x03, 0x3C, 0x2C, 0x1103);
		
#if USE_RunApplicationEventLoop
	EventLoopTimerUPP timer= NewEventLoopTimerUPP(one);
	
	InstallEventLoopTimer(GetMainEventLoop(), 0.0, 0.0, timer, launchOptions, NULL);
	
	RunApplicationEventLoop();
#else
	launchVM();
#endif
	
	return 0;
}

int launchVM()
{
    int i, result= 0;
    JNIEnv* env;
    JavaVM* theVM;
	
#if DEBUG
	char buf[200];
	fprintf(stderr, "wd: %s\n", getcwd(buf, 200));
	    
	fprintf(stderr, "options:\n");
	for (i= FirstOptionArg; i < LastOptionArg; i++)
		if (Args[i] != NULL)
			fprintf(stderr, "  <%s>\n", Args[i]);
		
	fprintf(stderr, "mainclass: %s\n", Args[MainClassArg]);

	fprintf(stderr, "args:\n");
	for (i= FirstMainArg; i < LastMainArg; i++)
		fprintf(stderr, "  <%s>\n", Args[i]);
#endif
	
    /* Set our name for the Application Menu to our Main class */
    char a[32];
    pid_t id= getpid();
    sprintf(a, "APP_NAME_%ld", (long)id);
    setenv(a, Args[MainClassArg], 1);
    
    /* default vm args */
    JavaVMInitArgs vm_args;
    
    /* JNI_VERSION_1_4 is used on Mac OS X to indicate the 1.4.x JVM */
    vm_args.version= JNI_VERSION_1_4;
    
	vm_args.ignoreUnrecognized= JNI_TRUE;
    vm_args.nOptions= OptionArgCount;
    vm_args.options= (JavaVMOption*) calloc(OptionArgCount, sizeof(JavaVMOption));
    int j= 0;
    for (i= FirstOptionArg; i < LastOptionArg; i++) {
    	char *option= (char*) Args[i];
    	if (option != NULL)
    		vm_args.options[j++].optionString= option;
    }
 
    /* start a VM session */    
    result= JNI_CreateJavaVM(&theVM, (void**)&env, &vm_args);

    if (result != 0) {
        fprintf(stderr, "%s: Error starting up VM.\n", ProgName);
        exit(result);
        return NULL;
    }
    
	/* Replace '.' with '/' in qualified main class name */
	char c, *cp= (char*) Args[MainClassArg];
	for (; c= *cp; cp++)
		if (c == '.')
			*cp= '/';
    
	/* Find the main class */
    jclass mainClass= (*env)->FindClass(env, Args[MainClassArg]);
    if (mainClass == NULL) {
        (*env)->ExceptionDescribe(env);
        result= -1;
        goto leave;
    }

    /* Get the application's main method */
    jmethodID mainID= (*env)->GetStaticMethodID(env, mainClass, "main", "([Ljava/lang/String;)V");
    if (mainID == NULL) {
        if ((*env)->ExceptionOccurred(env)) {
            (*env)->ExceptionDescribe(env);
        } else {
            fprintf(stderr, "%s: No main method found in specified class.\n", ProgName);
        }
        result= -1;
        goto leave;
    }

    /* Build argument array */
    const char **strv= &Args[FirstMainArg];
    int strc= LastMainArg-FirstMainArg;
    jarray mainArgs= NULL;
    jarray cls= (*env)->FindClass(env, "java/lang/String");
    if (cls != NULL) {
        mainArgs= (*env)->NewObjectArray(env, strc, cls, 0);
        if (mainArgs != NULL) {
     		int i;
            /* Add each of the c strings to the new array as UTF java.lang.String objects */
            for (i= 0; i < strc; i++) {
                jstring str= (*env)->NewStringUTF(env, *strv++);
                if (str != NULL) {
                    (*env)->SetObjectArrayElement(env, mainArgs, i, str);
                    (*env)->DeleteLocalRef(env, str);
                } else
                    break;
            }
		} else {
       	 	(*env)->ExceptionDescribe(env);
        	goto leave;
		}
    }
    
    /* Invoke main method passing in the argument object. */
    (*env)->CallStaticVoidMethod(env, mainClass, mainID, mainArgs);
    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        result= -1;
        goto leave;
    }
        
leave:
    (*theVM)->DestroyJavaVM(theVM);
    exit(result);
    
    // not reached
    return 0;
}
