/*******************************************************************************
 * Copyright (c) 2003 International Business Machines Corp. and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v0.5 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v05.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 ******************************************************************************/
package org.eclipse.jdt.internal.ui.refactoring;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;

import org.eclipse.jdt.internal.corext.refactoring.participants.xml.AndExpression;
import org.eclipse.jdt.internal.corext.refactoring.participants.xml.Expression;
import org.eclipse.jdt.internal.corext.refactoring.participants.xml.ExpressionParser;
import org.eclipse.jdt.internal.corext.refactoring.participants.xml.ITestResult;

public abstract class AbstractDescriptor {

	protected IConfigurationElement fConfigurationElement;
	protected Expression fExpression;
	
	protected static final String ID= "id"; //$NON-NLS-1$
	protected static final String OBJECT_STATE= "objectState";  //$NON-NLS-1$
	protected static final String CLASS= "class"; //$NON-NLS-1$
	
	protected AbstractDescriptor(IConfigurationElement element) {
		fConfigurationElement= element;
	}
	
	public String getId() {
		return fConfigurationElement.getAttribute(ID);
	}
	
	public boolean matches(Object element) throws CoreException {
		Expression exp= getExpression();
		if (exp.evaluate(element) == ITestResult.FALSE)
			return false;
		return true;
	}
	
	public Expression getExpression() {
		if (fExpression == null)
			fExpression= createExpression(fConfigurationElement);
		return fExpression;
	}
	
	public void clear() {
		fExpression= null;
	}
		
	protected Expression createExpression(IConfigurationElement element) {
		return createObjectStateExpression(element, Expression.FALSE);
	} 

	protected static Expression createObjectStateExpression(IConfigurationElement element, Expression defaultValue) {
		IConfigurationElement[] children= element.getChildren(OBJECT_STATE);
		if (children.length == 0) {
			return defaultValue;
		} else if (children.length == 1) {
			return ExpressionParser.getStandard().parse(children[0]);
		} else {
			AndExpression result= new AndExpression();
			for (int i= 0; i < children.length; i++) {
				result.add(ExpressionParser.getStandard().parse(children[i]));
			}
			return result;
		}
	}	
}
